@@ -36,10 +36,14 @@ import static org.junit.Assert.*;
 import static org.junit.Assume.assumeTrue;
 
 public class TreeDigestTest extends TDigestTest {
+    @BeforeClass
+    public static void setup() throws IOException {
+        TDigestTest.setup("tree");
+    }
 
-    private DigestFactory<TDigest> factory = new DigestFactory<TDigest>() {
+    private DigestFactory<TreeDigest> factory = new DigestFactory<TreeDigest>() {
         @Override
-        public TDigest create() {
+        public TreeDigest create() {
             return new TreeDigest(100);
         }
     };
@@ -49,13 +53,6 @@ public class TreeDigestTest extends TDigestTest {
         RandomUtils.useTestSeed();
     }
 
-    @After
-    public void flush() {
-        sizeDump.flush();
-        errorDump.flush();
-        deviationDump.flush();
-    }
-
     @Test
     public void testUniform() {
         Random gen = RandomUtils.getRandom();
@@ -133,10 +130,10 @@ public class TreeDigestTest extends TDigestTest {
         }
 
         System.out.printf("# %fus per point\n", (System.nanoTime() - t0) * 1e-3 / 100000);
-        System.out.printf("# %d centroids\n", dist.centroidCount());
+        System.out.printf("# %d centroids\n", dist.centroids().size());
 
         // I would be happier with 5x compression, but repeated values make things kind of weird
-        assertTrue("Summary is too large", dist.centroidCount() < 10 * (double) 1000);
+        assertTrue("Summary is too large", dist.centroids().size() < 10 * (double) 1000);
 
         // all quantiles should round to nearest actual value
         for (int i = 0; i < 10; i++) {
@@ -194,7 +191,7 @@ public class TreeDigestTest extends TDigestTest {
 
         buf.flip();
         TreeDigest dist2 = TreeDigest.fromBytes(buf);
-        assertEquals(dist.centroidCount(), dist2.centroidCount());
+        assertEquals(dist.centroids().size(), dist2.centroids().size());
         assertEquals(dist.compression(), dist2.compression(), 0);
         assertEquals(dist.size(), dist2.size());
 
@@ -216,7 +213,7 @@ public class TreeDigestTest extends TDigestTest {
 
         buf.flip();
         dist2 = TreeDigest.fromBytes(buf);
-        assertEquals(dist.centroidCount(), dist2.centroidCount());
+        assertEquals(dist.centroids().size(), dist2.centroids().size());
         assertEquals(dist.compression(), dist2.compression(), 0);
         assertEquals(dist.size(), dist2.size());
 
@@ -341,7 +338,7 @@ public class TreeDigestTest extends TDigestTest {
         assumeTrue(Boolean.parseBoolean(System.getProperty("runSlowTests")));
 
         final Random gen0 = RandomUtils.getRandom();
-        final PrintWriter out = new PrintWriter(new FileOutputStream("scaling.tsv"));
+        final PrintWriter out = new PrintWriter(new FileOutputStream("scaling-tree.tsv"));
         out.printf("k\tsamples\tcompression\tsize1\tsize2\n");
 
         List<Callable<String>> tasks = Lists.newArrayList();
@@ -474,17 +471,9 @@ public class TreeDigestTest extends TDigestTest {
         // answer to extreme quantiles in that case ('extreme' in the sense that the
         // quantile is either before the first node or after the last one)
         TreeDigest digest = new TreeDigest(100);
-        // we need to create the GroupTree manually
-        GroupTree tree = (GroupTree) digest.centroids();
-        Centroid g = new Centroid(10);
-        g.add(10, 2); // 10 has a weight of 3 (1+2)
-        tree.add(g);
-        g = new Centroid(20); // 20 has a weight of 1
-        tree.add(g);
-        g = new Centroid(40);
-        g.add(40, 4); // 40 has a weight of 5 (1+4)
-        tree.add(g);
-        digest.count = 3 + 1 + 5;
+        digest.add(10, 3);
+        digest.add(20, 1);
+        digest.add(40, 5);
         // this group tree is roughly equivalent to the following sorted array:
         // [ ?, 10, ?, 20, ?, ?, 50, ?, ? ]
         // and we expect it to compute approximate missing values:
